@@ -5,11 +5,13 @@ from . import voltage
 
 from .distributions import *
 from .waterfall_utils import *
-from .frame import Frame
+from .frame import *
 from .sample_from_obs import *
 from .split_utils import *
 from .stats import *
 from .unit_utils import *
 from .frame_utils import *
 from .normalize import *
-from .dedrift import *
\ No newline at end of file
+from .dedrift import *
+
+from .cadence import Cadence
\ No newline at end of file
